@@ -310,6 +310,13 @@ void KSelectionProxyModelTest::deselection_data()
             << 1
             << QStringList{"4"} << 5;
     ++testNumber;
+
+    QTest::newRow(QByteArray("test" + QByteArray::number(testNumber)).data())
+            << static_cast<int>(KSelectionProxyModel::ChildrenOfExactSelection)
+            << QStringList{"6", "7"} << 1
+            << 0
+            << QStringList{"7"} << 1;
+    ++testNumber;
 }
 
 void KSelectionProxyModelTest::deselection()
@@ -604,6 +611,13 @@ void KSelectionProxyModelTest::removeRows_data()
                   << 1
                   << QStringList{"9", "9"} << 2;
           ++testNumber;
+
+          QTest::newRow(QByteArray("test" + QByteArray::number(testNumber)).data())
+                  << static_cast<int>(kspm_mode) << connectSelectionModelFirst << false
+                  << QStringList{"6", "8", "11"} << 4
+                  << 0
+                  << QStringList{"8", "8"} << 4;
+          ++testNumber;
       }
   }
 
@@ -843,7 +857,7 @@ void KSelectionProxyModelTest::removeRows()
     {
         QObject::connect(&tree, &QAbstractItemModel::rowsAboutToBeRemoved, &tree,
                          [&tree, &selectionModel](QModelIndex const& p, int s, int e){
-            auto rmIdx = p.child(s, 0);
+            auto rmIdx = tree.index(s, 0, p);
             if (s == e && selectionModel.selectedIndexes().contains(rmIdx))
             {
                 auto nextIdx = tree.index(e + 1, 0, rmIdx.parent());
